package api.nextcloud

import co.appreactor.news.BuildConfig
import http.authInterceptor
import http.loggingInterceptor
import http.trustSelfSignedCerts
import okhttp3.OkHttpClient
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.util.concurrent.TimeUnit

class NextcloudApiBuilder {

    fun build(
        url: String,
        username: String,
        password: String,
        trustSelfSignedCerts: Boolean,
    ): NextcloudApi {
        val clientBuilder = OkHttpClient.Builder()
            .addInterceptor(authInterceptor(username, password))
            .connectTimeout(15, TimeUnit.SECONDS)
            .readTimeout(20, TimeUnit.SECONDS)
            .writeTimeout(20, TimeUnit.SECONDS)

        if (trustSelfSignedCerts) {
            clientBuilder.trustSelfSignedCerts()
        }

        if (BuildConfig.DEBUG) {
            clientBuilder.addInterceptor(loggingInterceptor("miniflux"))
        }

        return Retrofit.Builder()
            .baseUrl("$url/index.php/apps/news/api/v1-2/")
            .addConverterFactory(GsonConverterFactory.create())
            .client(clientBuilder.build())
            .build()
            .create(NextcloudApi::class.java)
    }
}